package markov;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

public class Lookup implements Serializable {

    private final LinkedHashMap<Token, Integer> tokens = new LinkedHashMap<Token, Integer>();
    private int totalCounts =0;
    int getTotalCounts() {
        return totalCounts;
    }

    int getDistinctTokens() {
        return tokens.size();
    }

    int getAmount(Token t) {
        return tokens.getOrDefault(t, 0);
    }

    public void add(Token t) {
        Integer i = tokens.getOrDefault(t, 0);
        tokens.put(t, i + 1);
        totalCounts++;
    }

    public Decission forRandom(double random) {
        if (random < 0 || random >= 1) {
            throw new IllegalArgumentException("expected double [0; 1)");
        }
        int id = (int) Math.floor(random * getTotalCounts());
        int i = 0;
        for (Entry<Token, Integer> entry : tokens.entrySet()) {
            i += entry.getValue();
            if (id < i) {
                return new Decission(entry.getKey(), id);
            }
        }
        throw new IllegalStateException(String.format(
                "failed to find a random token with seed %d within %s", id,
                this));

    }

    public Decission forId(int id) {
        if (id < 0 || id > getTotalCounts()) {
            throw new IllegalArgumentException(String.format("got %d but expected id [0; %d)",id, getTotalCounts()));
        }
        int i = 0;
        for (Entry<Token, Integer> entry : tokens.entrySet()) {
            i += entry.getValue();
            if (id < i) {
                return new Decission(entry.getKey(), id);
            }
        }
        throw new IllegalStateException(String.format(
                "failed to find a random token with seed %d within %s", id,
                this));

    }

    public Decission average() {
        return tokens
                .entrySet().stream().sorted((e1, e2) -> Integer
                        .compare(e2.getValue(), e1.getValue()))
                .findFirst().map(e -> new Decission(e.getKey(), e.getValue())).get();
    }

    public Set<Token> allPossible() {
        return this.tokens.keySet();
    }

    @Override
    public String toString() {
        return "Lookup [tokens= "
                + tokens.entrySet().stream()
                .sorted((e1, e2) -> Integer.compare(e2.getValue(),
                        e1.getValue()))
                .map(e -> String.format("%d*%s", e.getValue(),
                        e.getKey()))
                .collect(Collectors.joining(", "))
                + "]";
    }

}
